/*
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: buffer.h,v 1.4 2000/12/12 18:45:19 guenter Exp $
 *
 *
 * contents:
 *
 * buffer_entry structure - serves as a transport encapsulation
 *   of the mpeg audio/video data through xine
 *
 * free buffer pool management routines
 *
 * FIFO buffer structures/routines
 *
 */

#ifndef HAVE_BUFFER_H
#define HAVE_BUFFER_H

#include <pthread.h>
#include <inttypes.h>

#define BUF_MPEG2VIDEO 0
#define BUF_AC3AUDIO   1
#define BUF_MPEGAUDIO  2
#define BUF_MPEG1VIDEO 3
#define BUF_RESET      4
#define BUF_AVIVIDEO   5

typedef struct buf_element_s {
  struct buf_element_s *next;

  unsigned char        *pMem;
  unsigned char        *pContent; /* start of raw content in pMem (without header etc) */

  uint32_t              nSize ;   /* size of _content_ */
  uint32_t              nMaxSize;        
  uint32_t              nType;
  uint32_t              nPTS, nDTS;
  off_t                 nInputPos; /* remember where this buf came from in the input source */
} buf_element_t;

typedef struct fifo_buffer_s
{
  buf_element_t  *mpFirst, *mpLast;

  pthread_mutex_t mMutex;
  pthread_cond_t  mNotEmpty;

} fifo_buffer_t;

/*
 * init buffer pool, allocate nNumBuffers of buf_size bytes each 
 */

void buffer_pool_init (int nNumBuffers, uint32_t buf_size);

/*
 * check if there are no more free elements
 */

int buffer_pool_isempty ();

/*
 * allocate a buffer element from buffer pool
 */
buf_element_t *buffer_pool_alloc ();

/*
 * put a previously allocated buffer element back into the buffer pool
 */

void buffer_pool_free (buf_element_t *pBufElement) ;

/*
 * allocate and initialize new (empty) fifo buffer
 */

fifo_buffer_t *fifo_buffer_new ();

/*
 * append buffer element to fifo buffer
 */

void fifo_buffer_put (fifo_buffer_t *pFifo, buf_element_t *pBufElement);

/*
 * get element from fifo buffer
 */

buf_element_t *fifo_buffer_get (fifo_buffer_t *pFifo);

/*
 * buffer status
 */

int fifo_buffer_is_empty (fifo_buffer_t *pFifo);


/*
 * clear buffer (put all contained buffer elements back into buffer pool)
 */

void fifo_buffer_clear (fifo_buffer_t *pFifo) ;

#endif
