/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: configfile.c,v 1.3 2000/11/10 02:38:50 guenter Exp $
 *
 * config file management - implementation
 *
 */
 
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "configfile.h"

typedef struct cfg_entry_s {
  struct cfg_entry_s *next;
  char               *key, *value;
} cfg_entry_t;

cfg_entry_t *gConfig, *gConfigLast;

void config_file_add (char *key, char *value) {

  cfg_entry_t *entry;
  int          len;

  entry = (cfg_entry_t *) malloc (sizeof (cfg_entry_t));

  len = strlen (key);
  entry->key = (char *) malloc (len+2);
  strncpy (entry->key, key, len+1);

  len = strlen (value);
  entry->value = (char *) malloc (len+21);
  strncpy (entry->value, value, len+1);

  entry->next = NULL;

  if (gConfigLast) 
    gConfigLast->next = entry;
  else
    gConfig = entry;
  
  gConfigLast = entry;

}

void config_file_init () {
  FILE *f_config;
  char filename[1024];

  gConfig = NULL;
  gConfigLast = NULL;

  sprintf (filename, "%s/.xinerc", getenv ("HOME"));

  f_config = fopen (filename, "r");

  if (f_config) {

    char line[1024];
    char *value;

    while (fgets (line, 1023, f_config)) {
      line[strlen(line)-1]= (char) 0; /* eliminate lf */

      if (line[0] == '#')
	continue;
      
      if ((value = strchr (line, ':'))) {
	*value = (char) 0;
	value++;

	config_file_add (line, value);
      }

    }

    fclose (f_config);
  }
}

cfg_entry_t *config_file_search (char *key) {
  cfg_entry_t *entry;

  entry = gConfig;

  while (entry && strcmp (entry->key, key))
    entry = entry->next;

  return entry;
}

char *config_file_lookup_str (char *key, char*str_default) {
  cfg_entry_t *entry;

  entry = config_file_search (key);

  if (entry)
    return entry->value;

  config_file_add (key, str_default);

  return str_default;
}

int config_file_lookup_int (char *key, int n_default) {

  cfg_entry_t *entry;
  char str[25];

  entry = config_file_search (key);

  if (entry) {
    int n;

    if (sscanf (entry->value, "%d", &n) == 1) 
      return n;
  }

  sprintf (str, "%d", n_default);

  config_file_add (key, str);

  return n_default; 
}

void config_file_set_int (char *key, int value) {
  
  cfg_entry_t *entry;

  entry = config_file_search (key);

  if (entry) {
    sprintf (entry->value, "%d", value);
  } else {
    char str[25];
    sprintf (str, "%d", value);

    config_file_add (key, str);
  }
}

void config_file_set_str (char *key, char *value) {

  cfg_entry_t *entry;

  entry = config_file_search (key);

  if (entry) {
    int len;

    free (entry->value);

    len = strlen (value);
    entry->value = (char *) malloc (len+20);
    strncpy (entry->value, value, len);

  } else {
    config_file_add (key, value);
  }
}

void config_file_save () {
  FILE *f_config;
  char filename[1024];

  sprintf (filename, "%s/.xinerc", getenv ("HOME"));

  f_config = fopen (filename, "w");

  if (f_config) {
    
    cfg_entry_t *entry;

    fprintf (f_config, "#\n# xine config file\n#\n");

    entry = gConfig;

    while (entry) {
      fprintf (f_config, "%s:%s\n",entry->key,entry->value);
      entry = entry->next;
    }

    fclose (f_config);
  }  
}
