#ifndef HAVE_XINE_H
#define HAVE_XINE_H

#include "inttypes.h"
#include "input/input_plugin.h"

/*
 * player status constants:
 */

#define XINE_STOP      0 
#define XINE_FILLBUF   1 
#define XINE_PLAY      2 
#define XINE_PAUSE     3 
#define XINE_QUIT      4

/* nStatus : current xine status */
typedef void (*gui_status_callback_func_t)(int nStatus);

/*
 * init xine - call once at startup
 */

void xine_init (int no_xv, gui_status_callback_func_t gui_status_callback);

/*
 * open an mpeg stream and play it
 *
 * name : mrl to open
 * pos  : start position 0..65535
 *
 */

void xine_play (char *MRL, int pos);

/*
 * toggle pause mode
 */

void xine_pause ();

/*
 * stop playing
 */

void xine_stop ();

/*
 * exit xine
 */

void xine_exit ();

/*
 * get player status
 */

int xine_get_status ();

/*
 * return the current audio channel
 */

int xine_get_audio_channel (void);

/*
 * set desired audio channel
 */

void xine_select_audio_channel (int nChannel);

input_plugin_t* xine_get_input_plugin_list (int *nInputPlugins); 


/*
 * Get current position in stream
 */
int xine_get_current_position (void);


/*
 * Return current status (XINE_PLAY/XINE_STOP...)
 */
int xine_get_status(void);


/*
 * TODO / Ideas:
 *
 * eject
 * set volume / bright / contrast ...
 *
 * register callbacks
 */

/*
 * play(and more) from command line
 */
uint32_t  xine_user_pref;
#define PLAY_ON_START      0x00000001
#define PLAYED_ON_START    0x00000002
#define QUIT_ON_STOP       0x00000004
#define FULL_ON_START      0x00000008
#define HIDEGUI_ON_START   0x00000010

 
#endif
